# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_is_instance, assert_equal
from os import path
from wand.image import Image as WandImage

from assetman import init, create_blob, create_image
from assetman.core import Assetman
from assetman.core.assets import Asset, Image
from assetman.tests import TEST_KEY, TEST_SECRET, TEST_BUCKET

def test_init():
	am = init(TEST_KEY, TEST_SECRET, TEST_BUCKET)
	assert_is_instance(am, Assetman)

def test_create_blob():
	b = create_blob('some random data')
	assert_is_instance(b, Asset)

def test_create_image():
	with WandImage(filename=path.join(path.dirname(__file__), 'fixtures/faceoff.jpg')) as image:
		i = create_image(image.make_blob(), image.height, image.width, image.mimetype)
		assert_is_instance(i, Image)


# -*- coding: utf-8 -*-

from assetman.core import Assetman
from assetman.core.assets import Asset, Image


def init(key, secret, bucket):
	return Assetman(key, secret, bucket)

def create_blob(data):
	return Asset(data)

def create_image(data, height, width, mimetype):
	return Image(data, height, width, mimetype)



# -*- coding: utf-8 -*-

import logging

TEST_KEY = u'AKIAIUV632HFO2XMTROA'
TEST_SECRET = u'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'
TEST_BUCKET = u'opensource-test'

botoLogger = logging.getLogger('boto')
botoLogger.setLevel(logging.CRITICAL)


# -*- coding: utf-8 -*-

from assetman.service import s3
from assetman.core import assets

class Assetman(object):

	def __init__(self, key, secret, bucket):
		self._aws_key = key
		self._aws_secret = secret
		self._bucket_name = bucket
		self._connection = None

	@property
	def connection(self):
		if not self._connection:
			self._connection = s3.get_connection(self._aws_key, self._aws_secret)
		return self._connection

	def save_asset(self, asset):
		bucket = s3.get_bucket(self._bucket_name, self.connection)
		s3.upload_string_data(bucket, asset.uuid, asset.data, asset.public, asset.mime_type, asset.meta_data)
		return asset

	def get_asset(self, uuid):
		bucket = s3.get_bucket(self._bucket_name, self.connection)
		key = s3.get_key(bucket, uuid)
		if not key:
			return None
		
		asset = assets.lookup_by_type(key.get_metadata('asset_type'))(key.get_contents_as_string(), uuid)
		asset.url = key.generate_url(120)
		asset.mime_type = key.content_type
		asset.meta_data = key.metadata
		return asset

	def delete_asset(self, uuid):
		bucket = s3.get_bucket(self._bucket_name, self.connection)
		s3.delete_key(bucket, uuid)


# -*- coding: utf-8 -*-

import uuid

def lookup_by_type(asset_type):
	_asset_type_map = {
		'image': Image
	}

	try:
		return _asset_type_map[asset_type]
	except KeyError: pass
	
	return Asset

class Asset(object):

	def __init__(self, data=None, uuid=None):
		self.data = data
		self.mime_type = None
		self.meta_data = {}
		self.public = False
		self._uuid = uuid

	@property
	def uuid(self):
		if not self._uuid:
			self._uuid = uuid.uuid4()
		return self._uuid

	@property
	def asset_type(self):
		return self.meta_data['asset_type']

class Image(Asset):

	def __init__(self, data, height=0, width=0, mimetype=None):
		super(Image, self).__init__(data)
		self.mime_type = mimetype
		self.meta_data['height'] = height
		self.meta_data['width'] = width
		self.meta_data['asset_type'] = 'image'

	@property
	def height(self):
		return self.meta_data['height']

	@property
	def width(self):
		return self.meta_data['width']


# -*- coding: utf-8 -*-

from nose.tools.trivial import assert_is_none, assert_is_not_none, assert_is_instance, assert_equals

from boto.s3.connection import S3Connection

from assetman.tests import TEST_KEY, TEST_SECRET, TEST_BUCKET
from assetman.core import Assetman
from assetman.core.assets import Asset

def test_assetman():
	a = Assetman(TEST_KEY, TEST_SECRET, TEST_BUCKET)

	assert_is_none(a._connection)
	ctx = a.connection
	assert_is_instance(ctx, S3Connection)

	asset = Asset('my sweet data')
	a.save_asset(asset)
	assert_is_not_none(asset.uuid)

	retrieved = a.get_asset(asset.uuid)

	assert_equals(asset.uuid, retrieved.uuid)

	a.delete_asset(asset.uuid)

	assert_is_none(a.get_asset(asset.uuid))


# -*- coding: utf-8 -*-

from boto.s3.connection import S3Connection
from boto.s3.key import Key

def get_connection(key, secret):
	connection = S3Connection(key, secret)
	return connection

def get_bucket(bucket_name, connection):
	bucket = connection.get_bucket(bucket_name)
	return bucket

def upload_string_data(bucket, path, string_data, public=False, mime_type=None, metadata=None):
	key = Key(bucket, path)
	if mime_type:
		key.content_type = mime_type
	if metadata:
		key.update_metadata(metadata)
	key.set_contents_from_string(string_data)
	if public:
		key.make_public()

def delete_key(bucket, path):
	key = bucket.get_key(path)
	if key:
		key.delete()

def get_key(bucket, path):
	key = bucket.get_key(path)
	return key

def get_string_data(bucket, path):
	key = bucket.get_key(path)
	if key:
		return key.get_contents_as_string()


# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
	name='assetman',
	version='0.0.2',
	url='http://github.com/petermelias/assetman',
	license='BSD',
	author='Peter M. Elias',
	author_email='petermelias@gmail.com',
	description='Generic blob-data asset manager that provides an API, CLI interface, for managing S3 assets and the respective CloudFront delivery setup.',
	long_description=__doc__,
	packages=find_packages(),
	zip_safe=False,
	include_package_data=True,
	platforms='any',
	install_requires=[
		'boto',
	],
	extras_require={
		'test': ['nose', 'wand', 'coveralls']
	},
	classifiers=[
		'Environment :: Web Environment',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: BSD License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
		'Topic :: Software Development :: Libraries :: Python Modules'
	],
)